/******************************************************************************
 * $Header: /CommonBe/agmsmith/Programming/AGMSRAMFileSystem/RCS/AVLDupTree.h,v 1.16 2003/08/10 19:02:21 agmsmith Exp $
 *
 * This set of C subroutines is useful for indexing a set of key/value pairs,
 * using the key to find a matching value.  As an enhancement, multiple values
 * for the same key are supported efficiently.
 *
 * See the AVLDupTree.c file for API descriptions, license, algorithm and
 * contact details.  Copyright © 2001 by Alexander G. M. Smith.
 *
 * $Log: AVLDupTree.h,v $
 * Revision 1.16  2003/08/10 19:02:21  agmsmith
 * Changed time to be a signed 32 bit integer.
 *
 * Revision 1.15  2003/04/27 00:03:21  agmsmith
 * Add a unique code after the fake name used in indexes when seen as
 * directories, so that several entries in the index that have the same
 * key (such as files the same size in the size index) appear to be
 * different to old programs.  Also need to check if a particular key
 * and value are in the AVL tree.
 *
 * Revision 1.14  2003/01/12 00:51:57  agmsmith
 * Added external AVLDupGetComparisonFunctionForType function.
 *
 * Revision 1.13  2002/07/02 20:45:39  agmsmith
 * MIME strings are B_MIME_STRING_TYPE, not B_MIME_TYPE (whatever that is).
 *
 * Revision 1.12  2002/06/29 21:38:36  agmsmith
 * Added AVLDupGetThingTypeSize.
 *
 * Revision 1.11  2002/06/12 19:37:41  agmsmith
 * Added plus and minus infinity value test option for iteration.
 *
 * Revision 1.10  2002/06/04 20:47:54  agmsmith
 * Under construction, looking for date functions in kernel land.
 *
 * Revision 1.9  2002/05/31 23:28:31  agmsmith
 * Added a bunch of new data types for the key.
 *
 * Revision 1.8  2002/01/31 21:50:59  agmsmith
 * Changed values to be unsigned ints rather than void *, also
 * added AVLDupChangeValueInPlace and had to change the
 * iteration callback to use a pointer to the value not the value.
 *
 * Revision 1.7  2001/11/20 19:13:38  agmsmith
 * Added an error code for bad arguments when adding to the tree.
 *
 * Revision 1.6  2001/10/16 22:05:17  agmsmith
 * Changed return code from Add to specify separate codes for
 * adding a new node and trying to add an existing key/value.
 *
 * Revision 1.5  2001/07/29 20:06:05  agmsmith
 * Added a backwards iteration option and a FindPrevious function.
 *
 * Revision 1.4  2001/07/23 01:10:46  agmsmith
 * Added utility functions for quickly finding a key, basically
 * just a wrapper around the generic search function.
 *
 * Revision 1.3  2001/07/21 22:58:57  agmsmith
 * PoolID should be a const void *.
 *
 * Revision 1.2  2001/07/16 20:12:55  agmsmith
 * Changed to use a memory pool and to have a (void *) as the
 * value, and pass values by value, not by reference.
 *
 * Revision 1.1  2001/07/06 00:17:31  agmsmith
 * Initial revision
 *
 * Revision 1.11  2001/03/28 22:16:58  agmsmith
 * Prepared for 1.0 release.
 *
 * Revision 1.10  2001/03/22 22:58:13  agmsmith
 * Added double bounded iteration function, compiles but not tested yet.
 *
 * Revision 1.9  2001/03/18 17:53:23  agmsmith
 * Added semaphore for multitasking safety.
 *
 * Revision 1.8  2001/03/17 21:09:01  agmsmith
 * Printed it out, read it over, corrected a few mistakes.
 *
 * Revision 1.7  2001/03/15 18:02:13  agmsmith
 * Removed duplicates tree, partially working now without it.
 *
 * Revision 1.6  2001/03/12 22:38:36  agmsmith
 * Just before removing duplicates subtree, but it does work a bit.
 *
 * Revision 1.5  2001/03/10 17:59:25  agmsmith
 * Under construction.
 *
 * Revision 1.4  2001/03/08 20:37:20  agmsmith
 * Moved big comments and private data declarations to the new AVLDupTree.c file.
 *
 * Revision 1.3  2001/03/06 19:05:49  agmsmith
 * Clean up the documentation after re-reading it the next day.
 *
 * Revision 1.2  2001/03/05 22:02:54  agmsmith
 * More function prototypes and data structures fleshed out.
 *
 * Revision 1.1  2001/03/04 22:50:44  agmsmith
 * Initial revision
 */

#ifndef _AVL_DUP_TREE_H
#define _AVL_DUP_TREE_H 1

#include <SupportDefs.h>

#ifdef __cplusplus
extern "C" {
#endif


/* AVLDupThing is used for storing keys and values (well, not any more, this
version is hacked up to only use an unsigned int as the value).  The keys are
stored directly, except for strings, which have separately allocated buffers if
the string won't fit directly.  It's 8 bytes long, so it can be set to zero by
filling the int64Thing field with zero (so don't make it longer without fixing
up all the code that assumes that).  We currently support:
B_BOOL_TYPE (uint8 equivalent),
B_CHAR_TYPE (uint8 equivalent),
B_DOUBLE_TYPE,
B_FLOAT_TYPE,
B_INT64_TYPE,
B_INT32_TYPE,
B_INT16_TYPE,
B_INT8_TYPE,
B_MIME_STRING_TYPE (string equivalent),
B_OFF_T_TYPE (int64 equivalent),
B_SIZE_T_TYPE (uint32 equivalent),
B_SSIZE_T_TYPE (int32 equivalent),
B_STRING_TYPE,
B_TIME_TYPE (int32 equivalent),
B_UINT64_TYPE,
B_UINT32_TYPE,
B_UINT16_TYPE,
B_UINT8_TYPE. */

typedef union AVLDupThingUnion
{
  double    doubleThing;
  float     floatThing;
  int8      int8Thing;
  int16     int16Thing;
  int32     int32Thing;
  int64     int64Thing;
  time_t    timeThing;
  uint8     uint8Thing;
  uint16    uint16Thing;
  uint32    uint32Thing;
  uint64    uint64Thing;

  char      shortStringThing [8]; /* For shorter strings, [7] set to zero. */
  struct LongStringStruct
  {
    char   *stringPntr; /* String longer than 7 bytes + NUL. */
    uint8   filler1;
    uint8   filler2;
    uint8   filler3;
    uint8   isLongString; /* Non-zero for long strings, 0 for short ones. */
  } longStringThing;

} AVLDupThingRecord, *AVLDupThingPointer;

typedef const AVLDupThingRecord *AVLDupThingConstPointer;


/* A macro for getting a string pointer from a string thing.  Handles both long
and short string formats. */

#define AVLDupGetStringPntrFromThing(Thing) \
  ((Thing).longStringThing.isLongString \
  ? (Thing).longStringThing.stringPntr \
  : &(Thing).shortStringThing[0])  /* Added the &[0] to avoid type problems. */

/* Get size of a thing in bytes, 0 for variable sized, negative for error. */

int AVLDupGetThingTypeSize (type_code ThingType);

/* Comparison functions for the various different data types use this function
prototype.  It returns A - B in effect, so the result is >0 for A > B,
<0 for A < B and 0 for A == B. */

typedef int (* AVLDupComparisonFunctionPointer) (
  AVLDupThingConstPointer A, AVLDupThingConstPointer B);

AVLDupComparisonFunctionPointer AVLDupGetComparisonFunctionForType (
  type_code KeyType);


/* Memory allocation support functions for dealing with things.  Strictly
speaking you only need them for string things.  The memory pool for freeing a
Thing should be the same as the one used to allocate it. */

bool AVLDupCopyThingArray (
  AVLDupThingPointer DestThingPntr,
  AVLDupThingConstPointer SourceThingPntr,
  type_code ThingType,
  int NumberOfThings,
  void * (*MallocFunctionPntr) (const void *pool_id, size_t size),
  void (*FreeFunctionPntr) (const void *pool_id, void *ptr),
  const void *PoolID);

void AVLDupFreeThingArray (
  AVLDupThingPointer ThingPntr,
  type_code ThingType,
  int NumberOfThings,
  void (*FreeFunctionPntr) (const void *pool_id, void *ptr),
  const void *PoolID);

/* These convert between human readable strings and the binary formats.  For
the fancier data types, like dates, this can get quite complex. */

bool AVLDupConvertStringToThing (
  const char *StringPntr,
  AVLDupThingPointer ThingPntr,
  type_code ThingType,
  void * (*MallocFunctionPntr) (const void *pool_id, size_t size),
  void (*FreeFunctionPntr) (const void *pool_id, void *ptr),
  const void *PoolID);

bool AVLDupConvertThingToString (
  AVLDupThingConstPointer ThingPntr,
  type_code ThingType,
  char *StringPntr,
  int StringSize);


/* An opaque (to end users) reference to the AVLDupTree itself. */

typedef struct AVLDupTreeStruct AVLDupTreeRecord, *AVLDupTreePointer;


/* The various functions for operating on an AVLDupTree.  See the AVLDupTree.c
file for details on what they do and an explanation of the algorithms used. */

AVLDupTreePointer AVLDupAllocTree (
  type_code KeyType,
  void * (*MallocFunctionPntr) (const void *pool_id, size_t size),
  void (*FreeFunctionPntr) (const void *pool_id, void *ptr),
  const void *PoolID);

void AVLDupFreeTree (AVLDupTreePointer TreePntr);

unsigned int AVLDupGetTreeCount (AVLDupTreePointer TreePntr);

typedef enum AVLDupAddReturnCodesEnum {
  AVLADD_BAD_ARGUMENTS = -2,
  AVLADD_OUT_OF_MEMORY = -1,
  AVLADD_ADDED_A_NODE = 0,
  AVLADD_ALREADY_IN_TREE = 1
} AVLDupAddReturnCode;

AVLDupAddReturnCode AVLDupAdd (
  AVLDupTreePointer TreePntr,
  AVLDupThingPointer Key,
  unsigned int Value);

bool AVLDupDelete (
  AVLDupTreePointer TreePntr,
  AVLDupThingPointer Key,
  unsigned int Value);

typedef bool (* AVLDupIterationCallbackFunctionPointer) (
  AVLDupThingConstPointer KeyPntr,
  const unsigned int *ValuePntr,
  void *ExtraData);

bool AVLDupIterate (
  AVLDupTreePointer TreePntr,
  bool AscendingOrder,
  AVLDupThingConstPointer StartKeyPntr,
  int HowToCompareStartValue,
  unsigned int StartValue,
  bool IncludeThingEqualToStart,
  AVLDupThingConstPointer EndKeyPntr,
  int HowToCompareEndValue,
  unsigned int EndValue,
  bool IncludeThingEqualToEnd,
  AVLDupIterationCallbackFunctionPointer CallbackFunctionPntr,
  void *ExtraUserData);

unsigned int AVLDupFind (
  AVLDupTreePointer TreePntr,
  AVLDupThingConstPointer KeyPntr);

bool AVLDupKeyAndValueExists (
  AVLDupTreePointer TreePntr,
  AVLDupThingConstPointer KeyPntr,
  unsigned int Value);

unsigned int AVLDupFindNext (
  AVLDupTreePointer TreePntr,
  AVLDupThingConstPointer *KeyPntrPntr);

unsigned int AVLDupFindPrevious (
  AVLDupTreePointer TreePntr,
  AVLDupThingConstPointer *KeyPntrPntr);

bool AVLDupChangeValueInPlace (
  AVLDupTreePointer TreePntr,
  AVLDupThingConstPointer KeyPntr,
  unsigned int OldValue,
  unsigned int NewValue);

#ifdef __cplusplus
}
#endif

#endif /* _AVL_DUP_TREE_H */
